"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = exports.logError = exports.loggerMiddleware = void 0;
var process_1 = require("process");
var winston_1 = __importDefault(require("winston"));
var express_winston_1 = __importDefault(require("express-winston"));
var configs;
try {
    configs = require(process_1.cwd() + "/logger.config");
}
catch (_a) {
    throw Error('Logger configs could not be found on app root\'s directory.');
}
function formatReqResLog(info) {
    // in case the info itself is not from a request/response pair
    if (!info.meta || (info === null || info === void 0 ? void 0 : info.meta.req)) {
        return JSON.stringify(info);
    }
    var level = info.level, _a = info.meta, req = _a.req, res = _a.res, responseTime = _a.responseTime;
    var logData = Object.assign({}, { req: req, res: res, responseTime: responseTime + "ms" });
    return level + " - [" + req.method + "] " + req.url + ": " + JSON.stringify(logData);
}
function filterRequestHeaders(confidentialHeaders) {
    if (confidentialHeaders === void 0) { confidentialHeaders = []; }
    return function (req, propName) {
        if (propName !== "headers")
            return req[propName];
        var filteredHeaders = Object.keys(req.headers)
            .filter(function (key) { return !confidentialHeaders.includes(key); })
            .reduce(function (acc, key) {
            acc[key] = req.headers[key];
            return acc;
        }, {});
        return filteredHeaders;
    };
}
var routeFilter = function (req, res) {
    var url = req.url;
    var shouldRouteBeFiltered = shouldFilterRoute(url);
    return shouldRouteBeFiltered;
};
function shouldFilterRoute(url) {
    var routesToBeFiltered = configs.ignoredRoutes || [];
    var shouldRouteBeFiltered = routesToBeFiltered.some(function (route) {
        var splitedRouteParts = getSplitedRouteParts(route);
        return splitedRouteParts.every(function (part) { return url.toLowerCase().indexOf(part) > -1; });
    });
    return shouldRouteBeFiltered;
}
function getSplitedRouteParts(route) {
    var routeParts = route.split('/');
    var filteredParts = routeParts.filter(function (part) {
        var firstChar = part === null || part === void 0 ? void 0 : part.charAt(0);
        return firstChar !== ":" && firstChar !== '';
    });
    return filteredParts;
}
function createLoggerMiddleware() {
    var _a, _b;
    (_a = express_winston_1.default.requestWhitelist).push.apply(_a, __spreadArray(["body"], configs.requestWhitelist || []));
    (_b = express_winston_1.default.responseWhitelist).push.apply(_b, __spreadArray(["body"], configs.responseWhitelist || []));
    return express_winston_1.default.logger({
        statusLevels: true,
        transports: [
            new winston_1.default.transports.Console(configs.console),
        ],
        requestFilter: filterRequestHeaders(configs.confidentialHeaders || []),
        ignoreRoute: routeFilter,
        format: winston_1.default.format.combine(winston_1.default.format.colorize(), winston_1.default.format.printf(formatReqResLog)),
    });
}
function createErrorLoggerMiddleware() {
    return express_winston_1.default.errorLogger({
        transports: [
            new winston_1.default.transports.Console(configs.console)
        ],
        requestFilter: filterRequestHeaders(configs.confidentialHeaders || []),
    });
}
exports.loggerMiddleware = createLoggerMiddleware();
exports.logError = createErrorLoggerMiddleware();
exports.logger = winston_1.default.createLogger({
    transports: [
        new winston_1.default.transports.Console(configs.console)
    ],
    exitOnError: false,
});
exports.logger.stream = Object.assign(exports.logger.stream, {
    write: function (message) {
        exports.logger.info(JSON.parse(message));
    },
});
