"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApi = void 0;
var axios_1 = __importDefault(require("axios"));
var uuid_1 = require("uuid");
function createApi(config) {
    var axiosApi = axios_1.default.create(config);
    axiosApi.interceptors.request.use(function (request) {
        var data = request.data, params = request.params, method = request.method;
        var fullUrl = getFullUrlFromConfig(request);
        request.meta = request.meta || {};
        request.meta.requestId = uuid_1.v4();
        request.meta.requestStartedAt = new Date().getTime();
        logApiInfo(request.meta.requestId, 'Request', method, fullUrl, {
            data: data,
            params: params,
        });
        return request;
    });
    axiosApi.interceptors.response.use(function (response) {
        var status = response.status, data = response.data, config = response.config;
        var method = config.method, meta = config.meta;
        var fullUrl = getFullUrlFromConfig(config);
        var responseTime = getResponseTimeFromConfig(config);
        logApiInfo(meta === null || meta === void 0 ? void 0 : meta.requestId, 'Response', method, fullUrl, {
            status: status,
            data: data,
            responseTime: responseTime,
        });
        return response;
    }, function (error) {
        var config = error.config, message = error.message, response = error.response;
        var method = config.method, meta = config.meta;
        var fullUrl = getFullUrlFromConfig(config);
        var responseTime = getResponseTimeFromConfig(config);
        logApiInfo(meta === null || meta === void 0 ? void 0 : meta.requestId, 'Error', method, fullUrl, {
            status: response === null || response === void 0 ? void 0 : response.status,
            message: message,
            data: response === null || response === void 0 ? void 0 : response.data,
            responseTime: responseTime,
        });
        throw error;
    });
    return axiosApi;
}
exports.createApi = createApi;
function logApiInfo(id, title, method, url, content) {
    var formattedLog = "(" + title + ") " + id + " - [" + (method === null || method === void 0 ? void 0 : method.toUpperCase()) + "] " + url + ": " + JSON.stringify(content);
    console.log(formattedLog);
}
function getResponseTimeFromConfig(config) {
    var _a;
    var requestStart = (_a = config === null || config === void 0 ? void 0 : config.meta) === null || _a === void 0 ? void 0 : _a.requestStartedAt;
    var responseTime = '-';
    if (requestStart) {
        responseTime = String(new Date().getTime() - requestStart);
    }
    return responseTime + ' ms';
}
function getFullUrlFromConfig(config) {
    var baseURL = config.baseURL, url = config.url;
    var fullUrl = baseURL && url ? new String().concat(baseURL, url) : '';
    return fullUrl;
}
